/* main.c: description
 *
 * Copyright (C) 2004 Red Hat, Inc. All Rights Reserved.
 * Written by David Howells (dhowells@redhat.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/elf.h>
#include "module-verify.h"

extern int __init __ksign_init(void);
extern int __init __sha1_init(void);

int main(int argc, char *argv[])
{
	Elf_Ehdr *ehdr;

	struct stat st;
	void *module;
	int fd, ret;
	
	if (argc != 2) {
		fprintf(stderr, "Format: %s <module>\n", argv[0]);
		exit(2);
	}

	__sha1_init();
	__ksign_init();

	fd = open(argv[1], O_RDONLY);
	if (fd < 0) {
		perror("open");
		exit(1);
	}

	if (fstat(fd, &st) < 0) {
		perror("stat");
		exit(1);
	}

	module = mmap(NULL, st.st_size, PROT_READ|PROT_WRITE, MAP_PRIVATE, fd, 0);
	if (module == MAP_FAILED) {
		perror("mmap");
		exit(1);
	}

	close(fd);

	ehdr = module;

	ret = module_verify(ehdr, st.st_size);
	if (ret < 0) {
		errno = -ret;
		perror("module_verify");
		exit(1);
	}

	printf("Module verification succeeded (gpgsig_ok=%d)\n", ret);

	exit(0);
}
